@@ -1,10 +1,12 @@
 # -*- coding: utf-8 -*-
 
+from past.builtins import basestring
+
 from django.conf import settings
 from django.template import Library, Context, Template
 from django.utils.safestring import mark_safe
 from django.utils.http import urlencode
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 
 from django_stachoutils import urldecode
 from django_stachoutils.views.generic import ORDER_BY_ATTR, ORDER_TYPE_ATTR
@@ -37,6 +39,7 @@ def paginate_by_number(page, i, current_pagination, get_params={}):
     else:
         params = get_params.copy()
         params.update({PAGINATE_BY_VAR: i})
+        params = sorted([(k, v) for k, v in params.items()])
         return mark_safe(u'<a href="?%s"%s>%s</a> ' % (urlencode(params), '', str(i)))
 paginate_by_number = register.simple_tag(paginate_by_number)
 
@@ -44,7 +47,7 @@ paginate_by_number = register.simple_tag(paginate_by_number)
 # Librement inspiré de : http://code.djangoproject.com/browser/django/trunk/django/contrib/admin/templatetags/admin_list.py
 def pagination(page, current_pagination, paginates_by=[], get_params={}):
     paginates_by = paginates_by and (paginates_by + ['All']) or []
-    paginator, num_page = page.paginator, page.number
+    paginator = page.paginator
     pagination_required = (paginator.num_pages - 1)
     if not pagination_required:
         page_range = []
@@ -98,8 +101,6 @@ def processing(value):
 
 @register.filter
 def default_if_newrecord(record, default):
-   # if isinstance(record, ):
-   #     return value.strip()
     if record.pk:
         return record
     else:
@@ -133,7 +134,7 @@ def progressbar(value):
         total = value[1]
         perc_completed = 0
         if total > 0:
-            perc_completed = progress / total * 100
+            perc_completed = progress / total * 100  # FIXME with Integers.
             label = "%s/%s" % (progress, total)
     else:
         perc_completed = int(value)
@@ -196,15 +197,16 @@ def _table_header(columns, get_params, padding=0, sortable=True):
             link = u'%s' % c['label']
             if sortable:
                 order_type = 'asc'
-                order_by = _get_params.get(ORDER_BY_ATTR)
+                order_by = get_params.get(ORDER_BY_ATTR)
                 if order_by and order_by == str(i + padding):
                     style.append('sorted')
-                    if _get_params[ORDER_TYPE_ATTR] == 'asc':
+                    if get_params[ORDER_TYPE_ATTR] == 'asc':
                         style.append('ascending')
                         order_type = 'desc'
                     else:
                         style.append('descending')
                 _get_params.update({ORDER_BY_ATTR: i + padding, ORDER_TYPE_ATTR: order_type})
+                _get_params = sorted([(k, v) for k, v in _get_params.items()])
                 link = u'<a href="?%s">%s</a>' % (urlencode(_get_params), link)
             out.append(u'<th%s>%s<input class="fieldname" type="hidden" value="%s"/></th>' % (
                 (style and ' class="%s"' % ' '.join(style) or ''), link, c['field']
@@ -222,11 +224,11 @@ def table_row_tag(columns, instance):
 def _table_row(columns, instance):
     out = []
     for c in columns:
-        if c.has_key('with'):
+        if 'with' in c:
             out += _table_row(c['columns'], getattr(instance, c['with'])())
         else:
-            style, filters = [], u''
-            if c.has_key('label'):
+            style = []
+            if 'label' in c:
                 value = instance
                 # related lookup. On utilise pas qset.values() en amont qui eviterait ce truc récursif
                 # car empêche d'accéder des attr virtuels sur les instances.
@@ -246,13 +248,13 @@ def _table_row(columns, instance):
                 if c.get('tags'):
                     for tag in c['tags']:
                         value = tag(value)
-                if value and c.has_key('link'):
+                if value and 'link' in c:
                     href, title, a_style = "", "", ""
-                    if c['link'].has_key("href"):
+                    if 'href' in c['link']:
                         href = getattr(instance, c['link']['href'])()
-                    if c['link'].has_key("title"):
+                    if 'title' in c['link']:
                         title = getattr(instance, c['link']['title'])()
-                    if c['link'].has_key("style"):
+                    if 'style' in c['link']:
                         a_style = ' class="%s"' % c["link"]["style"]
                     value = u'<a%s title="%s" href="%s">%s</a>' % (a_style, title, href, value)
                 if c.get('editable'):
@@ -283,3 +285,33 @@ def get_template_filters(filters):
         else:
             t.append(u'|%s:"%s"' % (f[0], f[1]))
     return u''.join(t)
+
+
+## Inline-extras ##
+
+
+def inline_visible(inlines_visibles, inline_name, current_instance_number):
+    visible = True
+    current_instance_number = str(current_instance_number)
+    if inline_name in inlines_visibles:
+        visible = False
+        visibles = inlines_visibles[inline_name].split(",")
+        if current_instance_number in visibles:
+            visible = True
+    return visible
+
+
+@register.simple_tag
+def inline_display_class(inlines_visibles, inline_name, current_instance_number):
+    out = ""
+    if not inline_visible(inlines_visibles, inline_name, current_instance_number):
+        out = "closed"
+    return mark_safe(out)
+
+
+@register.simple_tag
+def inline_display_style(inlines_visibles, inline_name, current_instance_number):
+    out = ""
+    if not inline_visible(inlines_visibles, inline_name, current_instance_number):
+        out = "display: none;"
+    return mark_safe(out)
